@@ -31,6 +31,8 @@ class PostgreSQLModel(data_algebra.db_model.DBModel):
         op_replacements = data_algebra.db_model.db_default_op_replacements.copy()
         op_replacements['log'] = 'LN'
         op_replacements['_uniform'] = 'RANDOM'
+        op_replacements['std'] = 'STDDEV_SAMP'
+        op_replacements['var'] = 'VAR_SAMP'
         data_algebra.db_model.DBModel.__init__(
             self,
             identifier_quote='"',
