@@ -9,6 +9,7 @@ import copy
 import numpy
 import numbers
 import warnings
+import abc
 
 import sqlite3
 
@@ -48,9 +49,24 @@ def _sqlite_RAND_expr(dbmodel, expression):
     )
 
 
+def _sqlite_remainder_expr(dbmodel, expression):
+    """
+    Return SQL to check for bad values.
+    """
+
+    return (
+        "("
+        + dbmodel.expr_to_sql(expression.args[0], want_inline_parens=True)
+        + ' % '
+        + dbmodel.expr_to_sql(expression.args[1], want_inline_parens=True)
+        + ")"
+    )
+
+
 SQLite_formatters = {
     "is_bad": _sqlite_is_bad_expr,
     "rand": _sqlite_RAND_expr,
+    "remainder": _sqlite_remainder_expr,
 }
 
 
@@ -114,9 +130,9 @@ def _wrap_numpy_fn(f, x):
         return f([x])[0]
 
 
-class MedianAgg:
+class CollectingAgg(abc.ABC):
     """
-    Aggregate as median. SQLite user class.
+    Aggregate from a collection. SQLite user class.
     """
 
     def __init__(self):
@@ -129,13 +145,77 @@ class MedianAgg:
         if not _check_scalar_bad(value):
             self.collection.append(value)
 
+    @abc.abstractmethod
+    def calc(self) -> float:
+        """
+        Perform the calculation (only called with non-trivial data)
+        """
+
     def finalize(self):
         """
         Return result.
         """
         if len(self.collection) < 1:
+            res = numpy.nan
+        else:
+            res = self.calc()
+        self.collection = []
+        return res
+
+
+class MedianAgg(CollectingAgg):
+    """
+    Aggregate as median. SQLite user class.
+    """
+
+    def __init__(self):
+        CollectingAgg.__init__(self)
+
+    def calc(self) -> float:
+        "do it"
+        return float(numpy.median(self.collection))
+
+
+class SampVarDevAgg(CollectingAgg):
+    """
+    Aggregate as sample standard deviation. SQLite user class.
+    This version keeps the data instead of using the E[(x-E[x])^2] = E[x^2] - E[x]^2 formula
+    """
+
+    def __init__(self):
+        CollectingAgg.__init__(self)
+
+    def calc(self) -> float:
+        "do it"
+        # pandas.DataFrame({'x': [1., 3.], 'g': ['a', 'a']}).groupby(['g']).transform('var')['x']
+        # [2, 2]  # sample variance, what we want
+        # numpy.var([[1., 3.]])
+        # 1.0  # population variance- not what we want
+        n = len(self.collection)
+        if n < 2:
             return numpy.nan
-        return numpy.median(self.collection)
+        return float(numpy.var(self.collection) * n / (n-1))
+
+
+class SampStdDevAgg(CollectingAgg):
+    """
+    Aggregate as sample standard deviation. SQLite user class.
+    This version keeps the data instead of using the E[(x-E[x])^2] = E[x^2] - E[x]^2 formula
+    """
+
+    def __init__(self):
+        CollectingAgg.__init__(self)
+
+    def calc(self) -> float:
+        "do it"
+        # pandas.DataFrame({'x': [1., 3.], 'g': ['a', 'a']}).groupby(['g']).transform('std')['x']
+        # [1.4124214, 1.4124214]  # sample std deviation, what we want
+        # numpy.std([[1., 3.]])
+        # 1.0  # population std deviation- not what we want
+        n = len(self.collection)
+        if n < 2:
+            return numpy.nan
+        return float(numpy.std(self.collection) * numpy.sqrt(n / (n-1)))
 
 
 class SQLiteModel(data_algebra.db_model.DBModel):
@@ -258,6 +338,8 @@ class SQLiteModel(data_algebra.db_model.DBModel):
 
         # https://docs.python.org/3/library/sqlite3.html
         conn.create_aggregate("median", 1, MedianAgg)
+        conn.create_aggregate("std", 1, SampStdDevAgg)
+        conn.create_aggregate("var", 1, SampVarDevAgg)
 
     def _emit_right_join_as_left_join(
         self, join_node, *, using=None, temp_id_source, sql_format_options=None
