@@ -4,6 +4,7 @@ Base class for SQL adapters for data algebra.
 
 import math
 import re
+import warnings
 from collections import OrderedDict
 from typing import List, Optional
 from types import SimpleNamespace
@@ -19,6 +20,7 @@ import data_algebra.util
 import data_algebra.data_ops_types
 import data_algebra.data_ops
 from data_algebra.OrderedSet import OrderedSet
+import data_algebra.op_catalog
 
 
 # The db_model can be a bit tricky as SQL is represented a few ways depending
@@ -41,6 +43,8 @@ class SQLFormatOptions(SimpleNamespace):
         annotate: bool = True,
         sql_indent: str = " ",
         initial_commas: bool = False,
+        warn_on_method_support: bool = True,
+        warn_on_novel_methods: bool = True,
     ):
         assert isinstance(use_with, bool)
         assert isinstance(annotate, bool)
@@ -54,6 +58,8 @@ class SQLFormatOptions(SimpleNamespace):
             annotate=annotate,
             sql_indent=sql_indent,
             initial_commas=initial_commas,
+            warn_on_method_support=warn_on_method_support,
+            warn_on_novel_methods=warn_on_novel_methods,
         )
 
     def __str__(self):
@@ -609,6 +615,61 @@ db_default_op_replacements = {
 }
 
 
+def _annotated_method_catalogue(
+        model_name: str) -> Tuple[
+            Set[data_algebra.data_ops_types.MethodUse],
+            Set[data_algebra.data_ops_types.MethodUse]
+            ]:
+    """
+    Prepare method lookup tables
+
+    :param: model name (used as column key)
+    :return: (known_method_uses, recommended_method_uses)
+    """
+    known_method_uses: Set[data_algebra.data_ops_types.MethodUse] = set()
+    recommended_method_uses: Set[data_algebra.data_ops_types.MethodUse] = set()
+    for i in range(data_algebra.op_catalog.methods_table.shape[0]):
+        row = data_algebra.op_catalog.methods_table.loc[i, :]
+        matches = []
+        if row['op_class'] == 'p':
+            matches.append(data_algebra.data_ops_types.MethodUse(
+                row['op'],
+                is_project=True,
+                is_windowed=False,
+                is_ordered=False,
+            ))
+        elif row['op_class'] == 'g':
+            matches.append(data_algebra.data_ops_types.MethodUse(
+                row['op'],
+                is_project=False,
+                is_windowed=True,
+                is_ordered=False,
+            ))
+        elif row['op_class'] == 'w':
+            matches.append(data_algebra.data_ops_types.MethodUse(
+                row['op'],
+                is_project=False,
+                is_windowed=True,
+                is_ordered=True,
+            ))
+        else:
+            matches.append(data_algebra.data_ops_types.MethodUse(
+                row['op'],
+                is_project=False,
+                is_windowed=False,
+                is_ordered=False,
+            ))
+        if len(matches) > 0:
+            good_use = (
+                (model_name in data_algebra.op_catalog.methods_table.columns)
+                and (data_algebra.op_catalog.methods_table[model_name].values[i] == 'y'))
+            for m in matches:
+                known_method_uses.add(m)
+                if good_use:
+                    recommended_method_uses.add(m)
+    return known_method_uses, recommended_method_uses
+
+
 class DBModel:
     """A model of how SQL should be generated for a given database.
        """
@@ -625,6 +686,8 @@ class DBModel:
     default_SQL_format_options: SQLFormatOptions
     union_all_term_start: str
     union_all_term_end: str
+    known_methods: Optional[Set[data_algebra.data_ops_types.MethodUse]]
+    recommended_methods: Optional[Set[data_algebra.data_ops_types.MethodUse]]
 
     def __init__(
         self,
@@ -673,8 +736,16 @@ class DBModel:
         self.allow_extend_merges = allow_extend_merges
         self.union_all_term_start = union_all_term_start
         self.union_all_term_end = union_all_term_end
+        self.known_methods = None
+        self.recommended_methods = None
+        if str(self) in data_algebra.op_catalog.methods_table.columns:
+            k_meth, r_meth = _annotated_method_catalogue(str(self))
+            assert k_meth is not None  # type hint
+            assert r_meth is not None  # type hint
+            self.known_methods = k_meth
+            self.recommended_methods = r_meth
 
-    def db_handle(self, conn, db_engine=None):
+    def db_handle(self, conn, *, db_engine=None):
         """
 
         :param conn: database connection
@@ -911,12 +982,16 @@ class DBModel:
             op = expression.op
             if op in self.op_replacements.keys():
                 op = self.op_replacements[op]
-            elif op.lower() in  self.op_replacements.keys():
+            elif op.lower() in self.op_replacements.keys():
                 op = self.op_replacements[op.lower()]
+            elif op.upper() in self.op_replacements.keys():
+                op = self.op_replacements[op.upper()]
             if op in self.sql_formatters.keys():
                 return self.sql_formatters[op](self, expression)
             if op.lower() in self.sql_formatters.keys():
                 return self.sql_formatters[op.lower()](self, expression)
+            if op.upper() in self.sql_formatters.keys():
+                return self.sql_formatters[op.upper()](self, expression)
             if (len(expression.args) > 1) and expression.inline:
                 subs = [
                     self.expr_to_sql(ai, want_inline_parens=True)
@@ -1573,7 +1648,33 @@ class DBModel:
         )
         return near_sql
 
-    def to_sql(self, ops, *, sql_format_options=None) -> str:
+    def non_known_methods(
+            self,
+            ops: data_algebra.data_ops.ViewRepresentation) -> List[data_algebra.data_ops_types.MethodUse]:
+        """Return list of used non-recommended methods."""
+        if self.known_methods is None:
+            return []  # can't check, just pass
+        method_uses = ops.methods_used()
+        non_recommended = [op for op in method_uses if op not in self.known_methods]
+        non_recommended.sort()
+        return non_recommended
+
+    def non_recommended_methods(
+            self,
+            ops: data_algebra.data_ops.ViewRepresentation) -> List[data_algebra.data_ops_types.MethodUse]:
+        """Return list of used non-recommended methods."""
+        if (self.recommended_methods is None) or (self.known_methods is None):
+            return []  # can't check, just pass
+        method_uses = ops.methods_used()
+        non_recommended = [op for op in method_uses if
+                           (op not in self.recommended_methods) and (op in self.known_methods)]
+        return non_recommended
+
+    def to_sql(
+            self,
+            ops: data_algebra.data_ops.ViewRepresentation,
+            *,
+            sql_format_options: Optional[SQLFormatOptions] = None) -> str:
         """
         Convert ViewRepresentation into SQL string.
         """
@@ -1583,6 +1684,14 @@ class DBModel:
             sql_format_options = self.default_SQL_format_options
         assert isinstance(sql_format_options, SQLFormatOptions)
         ops.columns_used()  # for table consistency check/raise
+        if sql_format_options.warn_on_novel_methods:
+            non_known = self.non_known_methods(ops)
+            if len(non_known) > 0:
+                warnings.warn(f"{self} translation using undocumented method context: {non_known}", UserWarning)
+        if sql_format_options.warn_on_method_support:
+            non_rec = self.non_recommended_methods(ops)
+            if len(non_rec) > 0:
+                warnings.warn(f"{self} translation doesn't fully support method context: {non_rec}", UserWarning)
         temp_id_source = [0]
         near_sql = ops.to_near_sql_implementation_(
             db_model=self, using=None, temp_id_source=temp_id_source
@@ -2127,7 +2236,11 @@ class DBHandle:
         )
         return self.describe_table(table_name)
 
-    def to_sql(self, ops, *, sql_format_options=None) -> str:
+    def to_sql(
+            self,
+            ops: data_algebra.data_ops.ViewRepresentation,
+            *,
+            sql_format_options: Optional[SQLFormatOptions] = None) -> str:
         """
         Convert operations into SQL
         """
