@@ -2,8 +2,8 @@
 Represent data processing expressions.
 """
 
-from abc import ABC
-from typing import Any, List, Optional, Union
+import abc
+from typing import Any, List, Optional, Set, Union
 
 import numpy
 
@@ -104,6 +104,13 @@ fn_names_not_allowed_in_project = {
 }
 
 
+# fns that don't have consistent windowed implementations we want to support
+fn_names_that_contradict_windowed_situation = {
+    'std',
+    'var',
+}
+
+
 # a competing idea should be to remove ordering if
 # operator is one of these (instead of forbid)
 # noinspection SpellCheckingInspection
@@ -113,10 +120,12 @@ fn_names_that_contradict_ordered_windowed_situation = {
     "min",
     "prod",
     "sum",
+    'std',
+    'var',
 }
 
 
-class PreTerm(ABC):
+class PreTerm(abc.ABC):
     """
     abstract base class, without combination ability
     """
@@ -126,47 +135,56 @@ class PreTerm(ABC):
     def __init__(self):
         self.source_string = None
 
+    @abc.abstractmethod
     def is_equal(self, other):
         """
         Check if this expression code is the same as another expression.
         """
         # can't use == as that builds a larger expression
-        raise NotImplementedError("base method called")
 
     # tree re-write
 
+    @abc.abstractmethod
     def get_views(self):
         """
         return list of unique views, expectation list is of size zero or one
         """
-        raise NotImplementedError("base class called")
 
+    @abc.abstractmethod
     def replace_view(self, view):
         """
         Move expression to a new view.
         """
-        raise NotImplementedError("base class called")
 
     # analysis
 
-    def get_column_names(self, columns_seen):
+    def get_column_names(self, columns_seen: Set[str]) -> None:
         """
         Add column names to columns_seen
         :param columns_seen: set of strings
-        :return:
+        :return: None
+        """
+        pass
+
+    def get_method_names(self, methods_seen: Set[str]) -> None:
+        """
+        Add method names to methods_seen
+        :param methods_seen: set of strings
+        :return: None
         """
         pass
 
     # eval
 
+    @abc.abstractmethod
     def evaluate(self, data_frame):
         """
         Evaluate expression, taking data from data_frame.
         """
-        raise NotImplementedError("base class called")
 
     # emitters
 
+    @abc.abstractmethod
     def to_python(self, *, want_inline_parens: bool = False) -> PythonText:
         """
         Convert parsed expression into a string
@@ -174,9 +192,6 @@ class PreTerm(ABC):
         :param want_inline_parens: bool, if True put parens around complex expressions that don't already have a grouper.
         :return: PythonText
         """
-        raise NotImplementedError(
-            "base class method called"
-        )  # https://docs.python.org/3/library/exceptions.html
 
     def to_source(self, *, want_inline_parens=False, dialect="Python") -> PythonText:
         """
@@ -240,7 +255,7 @@ def _check_expr_incompatible_types(a, b):
 
 
 # noinspection SpellCheckingInspection
-class Term(PreTerm, ABC):
+class Term(PreTerm, abc.ABC):
     """
     Abstract intermediate class with combination ability
     """
@@ -431,7 +446,7 @@ class Term(PreTerm, ABC):
         """
         Return trigometric arctan2() (in radians) of item (vectorized).
         """
-        return self.__op_expr__("arctan2", other)
+        return self.__op_expr__("arctan2", other, inline=False, method=True)
 
     def sinh(self):
         """
@@ -533,7 +548,7 @@ class Term(PreTerm, ABC):
         """
         Return modulo of items (vectorized).
         """
-        return self.__op_expr__("mod", other)
+        return self.__op_expr__("mod", other, inline=False, method=True)
 
     def remainder(self, other):
         """
@@ -704,13 +719,13 @@ class Term(PreTerm, ABC):
 
     def std(self):
         """
-        Return standard devaition (vectorized).
+        Return sample standard devaition (vectorized).
         """
         return self.__uop_expr__("std")
 
     def var(self):
         """
-        Return variance (vectorized).
+        Return sample variance (vectorized).
         """
         return self.__uop_expr__("var")
 
@@ -1075,7 +1090,7 @@ class ListTerm(PreTerm):
             is_in_parens=False,
         )
 
-    def get_column_names(self, columns_seen):
+    def get_column_names(self, columns_seen: Set[str]) -> None:
         """
         Add column names to columns_seen
         :param columns_seen: set of strings
@@ -1141,14 +1156,6 @@ class DictTerm(PreTerm):
         ]
         return PythonText("{" + ", ".join(terms) + "}", is_in_parens=False)
 
-    def get_column_names(self, columns_seen):
-        """
-        Add column names to columns_seen
-        :param columns_seen: set of strings
-        :return:
-        """
-        pass
-
 
 def enc_value(value):
     """
@@ -1223,7 +1230,7 @@ class ColumnReference(Term):
         """
         return PythonText(self.column_name, is_in_parens=False)
 
-    def get_column_names(self, columns_seen):
+    def get_column_names(self, columns_seen: Set[str]) -> None:
         """
         Add column names to columns_seen
         :param columns_seen: set of strings
@@ -1268,13 +1275,6 @@ def _can_find_method_by_name(op):
             return True
     except AttributeError:
         pass
-    # new see if numpy can do this
-    try:
-        fn = numpy.__dict__[op]
-        if callable(fn):
-            return True
-    except KeyError:
-        pass
     return False
 
 
@@ -1282,7 +1282,7 @@ class Expression(Term):
     """
     Class for carrying an expression.
     """
-    def __init__(self, op, args, *, params=None, inline=False, method=False):
+    def __init__(self, op: str, args, *, params=None, inline: bool = False, method: bool = False):
         assert isinstance(op, str)
         if not _can_find_method_by_name(op):
             raise KeyError(f"can't find implementation for function/method {op}")
@@ -1348,7 +1348,7 @@ class Expression(Term):
             method=self.method,
         )
 
-    def get_column_names(self, columns_seen):
+    def get_column_names(self, columns_seen: Set[str]) -> None:
         """
         Add column names to columns_seen
         :param columns_seen: set of strings
@@ -1357,6 +1357,15 @@ class Expression(Term):
         for a in self.args:
             a.get_column_names(columns_seen)
 
+    def get_method_names(self, methods_seen: Set[str]) -> None:
+        """
+        Add names of methods used to methods_seen.
+
+        :param methods_seen: set to collect results
+        :return: None
+        """
+        methods_seen.add(self.op)
+
     def evaluate(self, data_frame):
         """
         Evaluate expression, taking data from data_frame.
@@ -1467,19 +1476,19 @@ def standardize_join_type(join_str):
 
 
 # noinspection SpellCheckingInspection
-def get_columns_used(parsed_exprs):
+def get_columns_used(parsed_exprs) -> Set[str]:
     """
     Return set of columns used in an expression.
     """
     assert isinstance(parsed_exprs, dict)
-    columns_seen = set()
+    columns_seen: Set[str] = set()
     for node in parsed_exprs.values():
         node.get_column_names(columns_seen)
     return columns_seen
 
 
 # noinspection SpellCheckingInspection
-def implies_windowed(parsed_exprs):
+def implies_windowed(parsed_exprs: dict) -> bool:
     """
     Return true if expression implies a windowed calculation is needed.
     """
